# -*- coding: utf-8 -*-
from unittest import TestCase

from django_settings_custom import encryption


class TestEncryption(TestCase):
    def test_encryption_can_be_decrypt(self):
        secret_key = "b(l$!na5rzuo+h(psyrlees(p9talk)u!=pjk#6=0v!n*q#y!m"
        source = "A protected sentence !"

        encrypted_source = encryption.encrypt(source, secret_key)
        self.assertNotEqual(source, encrypted_source)

        decrypted_source = encryption.decrypt(encrypted_source, secret_key)
        self.assertEqual(source, decrypted_source)



# -*- coding: utf-8 -*-
import os
import re

from six.moves.configparser import ConfigParser

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.management.utils import get_random_secret_key

from django_settings_custom import encryption


class Command(BaseCommand):
    """
    A Django interactive command for configuration file generation.

    Example:
        python manage.py generate_settings path/to/template/settings.ini target/path/of/settings.ini'

    Attributes:
        settings_template_file (str): Path to the settings template file.
        settings_file_path (str): Target path for the created settings file.
        force_secret_key (bool): Generate SECRET_KEY without asking ?
    """

    help = "A Django interactive command for configuration file generation."
    usage = "python manage.py generate_settings path/to/template/settings.ini target/path/of/settings.ini"

    settings_template_file = (
        settings.SETTINGS_TEMPLATE_FILE
        if hasattr(settings, "SETTINGS_TEMPLATE_FILE")
        else None
    )
    settings_file_path = (
        settings.SETTINGS_FILE_PATH if hasattr(settings, "SETTINGS_FILE_PATH") else None
    )
    force_secret_key = (
        settings.SETTINGS_FILE_PATH
        if hasattr(settings, "SETTINGS_FORCE_SECRET_KEY")
        else False
    )

    def add_arguments(self, parser):
        """
        Add custom arguments to the command.
        See python manage.py generate_settings --help.
        """
        parser.usage = self.usage
        parser.add_argument(
            "settings_template_file",
            nargs="?",
            type=str,
            default=self.settings_template_file,
            help="Path to the settings template file.",
        )
        parser.add_argument(
            "settings_file_path",
            nargs="?",
            type=str,
            default=self.settings_file_path,
            help="Target path for the settings file.",
        )
        parser.add_argument(
            "--force-secretkey",
            action="store_true",
            dest="force_secretkey",
            help="Generate SECRET_KEY without asking.",
        )

    @staticmethod
    def get_value(section, key, value_type, secret_key):
        """
        Get a value for the [section] key passed as parameter.

        Args:
            section (str): Section in the configuration file.
            key (str): Key in the configuration file.
            value_type (str): Value type read in template,
                must be "DJANGO_SECRET_KEY", "USER_VALUE" or "ENCRYPTED_USER_VALUE".
            secret_key: Django secret key

        Returns:
            int or str: Value for the [section] key
        """
        value = None
        if value_type == "DJANGO_SECRET_KEY":
            value = secret_key
        elif "USER_VALUE" in value_type:
            encrypt = value_type == "ENCRYPTED_USER_VALUE"
            encrypt_message = ""
            if encrypt:
                encrypt_message = "(will be encrypted) "
            value = input("Value for [%s] %s %s: " % (section, key, encrypt_message))
            if encrypt:
                value = encryption.encrypt(value, secret_key)
        return value

    def handle(self, *args, **options):
        """
        Command core.
        """
        settings_template_file = options["settings_template_file"]
        settings_file_path = options["settings_file_path"]
        force_secret_key = options.get("force_secretkey", self.force_secret_key)
        if not settings_template_file:
            raise CommandError(
                "Parameter settings_template_file undefined.\nUsage: %s" % self.usage
            )
        if not settings_file_path:
            raise CommandError(
                "Parameter settings_file_path undefined.\nUsage: %s" % self.usage
            )
        if not os.path.exists(settings_template_file):
            raise CommandError("The settings template file doesn't exists.")

        self.stdout.write("** Configuration file generation: **")
        if os.path.exists(settings_file_path):
            override = input(
                "A configuration file already exists at %s. "
                "Would you override it ? (y/N) : " % settings_file_path
            )
            if override.upper() != "Y":
                raise CommandError("Generation cancelled.")

        config = ConfigParser()
        config.optionxform = str
        config.read(settings_template_file)

        input_secret_key = False
        if not force_secret_key:
            generate_secret_key = input(
                "Do you want to generate the secret key for Django ? (Y/n) : "
            )
            input_secret_key = generate_secret_key.upper() == "N"
        if input_secret_key:
            secret_key = input("Enter your secret key : ")
            if not secret_key:
                raise CommandError(
                    "Django secret key is needed for encryption. Generation cancelled."
                )
        else:
            secret_key = get_random_secret_key()
            self.stdout.write("Django secret key generated !")
        secret_key = secret_key.replace("%", "0")

        self.stdout.write("\n** Filling values for configuration file content **")
        variable_regex = re.compile(r" *{(.+)} *")
        for section, values in config.items():
            for key, value in values.items():
                match_groups = variable_regex.match(value)
                if match_groups:
                    value_type = match_groups.group(1).strip().upper()
                    config.set(
                        section,
                        key,
                        self.get_value(section, key, value_type, secret_key),
                    )
        self.stdout.write("\nWriting file at %s:" % settings_file_path)
        os.makedirs(os.path.dirname(settings_file_path), exist_ok=True)
        with open(settings_file_path, "w") as config_file:
            config.write(config_file)
        self.stdout.write(
            self.style.SUCCESS("Configuration file successfully generated !")
        )



# -*- coding: utf-8 -*-
"""
.. module:: encryption
   :synopsis: Module to encrypt / decrypt values.
"""

import base64

import six
from Crypto import Random
from Crypto.Cipher import AES
from Crypto.Hash import SHA256

from django.conf import settings


def _compute_key(secret_key=None):
    """
    Compute a valid key for AES crypto algorithm.

    Args:
        secret_key (str): The key for encryption, or None if you want use the SECRET_KEY.

    Returns:
        byte string: A valid key for AES.

    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    """
    if secret_key is None:
        secret_key = settings.SECRET_KEY
    return SHA256.new(six.b(secret_key)).digest()


def encrypt(source, secret_key=None):
    """
    Encrypt the source with the key passed as parameter.

    Args:
        source (str or byte string): A string or a bytes array to encrypt.
        secret_key (str): The key for encryption, or None if you want use the SECRET_KEY.

    Returns:
        str: Encrypted value.

    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    """
    if isinstance(source, six.string_types):
        source = source.encode()
    key = _compute_key(secret_key)
    iv = Random.new().read(AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    padding = AES.block_size - len(source) % AES.block_size
    source += bytearray([padding]) * padding
    data = iv + cipher.encrypt(source.decode())
    return base64.b64encode(data).decode("latin-1")


def decrypt(source, secret_key=None):
    """
    Decrypt the source with the key passed as parameter.

    Args:
        source (str or byte string): A string or a bytes array to decrypt.
        secret_key (str): The key for encryption, or None if you want use the SECRET_KEY.

    Returns:
        str: Decrypted value.

    If the secret_key is not provided, it uses Django settings.SECRET_KEY.
    """
    key = _compute_key(secret_key)
    source = base64.b64decode(source.encode("latin-1"))
    iv = source[: AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    data = cipher.decrypt(source[AES.block_size :]).decode("utf-8")
    padding = ord(data[-1])
    if data[-padding:] != (bytearray([padding]) * padding).decode("utf-8"):
        raise ValueError("Error in decryption.")
    return data[:-padding]



# -*- coding: utf-8 -*-
import os

import yaml
from setuptools import find_packages, setup

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements

base_path = os.path.dirname(os.path.abspath(__file__))
requirements = [
    str(ir.req)
    for ir in parse_requirements(
        os.path.join(base_path, "requirements", "base.txt"), session=False
    )
]
requirements_test = [
    str(ir.req)
    for ir in parse_requirements(
        os.path.join(base_path, "requirements", "test.txt"), session=False
    )
]
requirements_dev = [
    str(ir.req)
    for ir in parse_requirements(
        os.path.join(base_path, "requirements", "dev.txt"), session=False
    )
]
extras_require = {
    "test": [r for r in requirements_test if r not in requirements],
    "dev": [r for r in requirements_dev if r not in requirements],
}

long_description = """Provide a Django interactive command for configuration file generation.
  See the project page for more information:
  https://github.com/ThomasMarques/django-settings-custom"""
if os.path.isfile("README.md"):
    with open("README.md") as f:
        long_description = f.read()


def get_version_info():
    from version import __version__

    return __version__


with open(".travis.yml") as travis_file:
    compatibility_information = yaml.load(travis_file)

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Web Environment",
    "Framework :: Django",
]
for django_version in compatibility_information["env"]:
    classifiers.append("Framework :: Django :: %s" % django_version.split("=")[1])
classifiers.extend(
    [
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ]
)
python_versions = set()
for python_version in compatibility_information["python"]:
    python_versions.add(python_version[:1])
    python_versions.add(python_version)
classifiers.extend(
    sorted(["Programming Language :: Python :: %s" % v for v in python_versions])
)
classifiers.extend(
    [
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Utilities",
    ]
)

setup(
    name="django-settings-custom",
    version=get_version_info(),
    description="A Django interactive command for configuration file generation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Thomas Marques",
    author_email="marquesthom@gmail.com",
    license="MIT License",
    platforms=["any"],
    url="https://github.com/ThomasMarques/django-settings-custom",
    packages=find_packages(where="."),
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    classifiers=classifiers,
)
